/*
 * This file is part of SubLink.
 * Copyright (C) 2009  Jeferson Hultmann <hultmann@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

"use strict";

var EXPORTED_SYMBOLS = ["AsyncRequest"];

Components.utils.import("resource://sublink-migreme-4.1.0/util.js");


function AsyncRequest() {
  this.requestUrl = null;
  this.onError = null;
  this.onSuccess = null;
  this.onComplete = null;
}

AsyncRequest.prototype = {

  _createXhr: function() {
    var xhr = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);
    xhr.mozBackgroundRequest = true;

    xhr.open("GET", this.requestUrl, true);
    xhr.setRequestHeader("Accept", "text/xml");
    xhr.setRequestHeader("Accept-Language", null);
    xhr.setRequestHeader("Accept-Encoding", null);
    xhr.setRequestHeader("Cookie", null);
    xhr.setRequestHeader("User-Agent", 'SubLink/4.1.0 (+http://br.mozdev.org/migre.me/)');
    // LOAD_ONLY_IF_MODIFIED makes xhr results from cache be empty!
    //xhr.channel.loadFlags |= Ci.nsICachingChannel.LOAD_ONLY_IF_MODIFIED;
    return xhr;
  },


  convert: function(uri) {
    this.requestUrl = "http://migre.me/api.xml?url=" + encodeURIComponent(uri.spec);
    //this.requestUrl = "http://localhost:81/migre.me/api-test.html?url=" + encodeURIComponent(uri.spec);

    var xhr = this._createXhr();
    var me = this;
    xhr.onload = function(evt) {
      me.onComplete();
      me.requestSuccess(this); // this == evt.target == evt.currentTarget == xhr;
    }
    xhr.onerror = function(evt) {
      me.onComplete();
      me.requestFail(this);
    }
    xhr.send(null);
  },

  // TODO nao + usado?
  findRedir: function(uri) {
    var idx = uri.path.indexOf('/', 1);
    if (idx > -1) {
      if (uri.path.indexOf('/', idx + 1) == -1) {
        var uri = uri.clone();
        uri.path = uri.path.substr(idx);
      }
    }


    this.requestUrl = "http://migre.me/api_redirect2.xml?url=" + encodeURIComponent(uri.spec);
    var xhr = this._createXhr();
    var me = this;
    xhr.onload = function(evt) {
      me.onComplete();
      me.requestSuccess(this); // this == evt.target == evt.currentTarget == xhr;
    }
    xhr.onerror = function(evt) {
      me.onComplete();
      me.requestFail(this);
    }
    xhr.send(null);
  },


  requestFail: function(xhr) {
    var errorData = {
      id: "noconnection",
      xhr: xhr,
      msg: null
    };
    this.onError(errorData);
  },


  requestSuccess: function(xhr) {
    var errorData = {
      id: "noconnection",
      xhr: xhr,
      msg: null
    };

    if (xhr.status !== 200) {
      errorData.id = "invalid";
      this.onError(errorData);
      return;
    }

    var doc = xhr.responseXML;
    if (doc === null) {
      errorData.id = "invalid";
      this.onError(errorData);
      return;
    }


    if (doc.documentElement.namespaceURI === "http://www.mozilla.org/newlayout/xml/parsererror.xml") {
      errorData.id = "malformed";
      this.onError(errorData);
      return;
    }


    var responseData = this.populateResponseData(doc);
    responseData.responseStatusCode = 301;


    //if (this.requestUrl.indexOf("http://localhost:81/migre.me") !== 0) {
    if (this.requestUrl.indexOf("http://migre.me/api.xml") !== 0) {
      // request to migre.me
      if (this.validateRedirData(responseData, errorData)) {
        var uri = util.stringToUri(responseData.longUrl);
        if (uri === null) {
          this.onError({
            id: "invalid",
            xhr: xhr,
            msg: null
          });
        } else {
          responseData.longUri = uri;
          this.onSuccess(responseData);
        }
      } else {
        this.onError(errorData);
      }

    } else {
      // request to a different server
      if (this.validateCompressResponse(responseData, errorData)) {
        var uri = util.stringToUri(responseData.shortUrl);
        if (uri === null) {
          this.onError({
            id: "invalid",
            xhr: xhr,
            msg: null
          });
        } else {
          this.onSuccess({
            shortUri:           uri,
            responseStatusCode: xhr.status
          });
        }
      } else {
        this.onError(errorData);
      }
    }
  },


  populateResponseData: function(doc) {

    var selectors = [":root > error",
                     ":root > errormessage",
                     ":root > migre",
                     ":root > url",
                     ":root > content_type",
                     ":root > title_link"];

    var properties = ["errorCode",
                      "errorMessage",
                      "shortUrl",
                      "longUrl",
                      "contentType",
                      "htmlTitle"];

    var rv = {};
    properties.forEach(function(val, idx) {
      var node = doc.querySelector(selectors[idx]);
      if (node) {
        rv[val] = node.textContent;
      }
    });

    return rv;
  },


  validateCompressResponse: function(responseData, errorData) {
    if (responseData.errorCode === null) {
      errorData.id = "invalid";
      return false;
    }
    if (responseData.errorCode !== "0") {
      errorData.id = "couldntconvert";
      errorData.msg = responseData.errorMessage === null ? "" : responseData.errorMessage;
      return false;
    }

    if (responseData.shortUrl === null) {
      errorData.id = "invalid";
      return false;
    }

    return true;
  },


  validateRedirData: function(responseData, errorData) {
    if (responseData.errorCode === null) {
      errorData.id = "invalid";
      return false;
    }
    if (responseData.errorCode !== "0") {
      errorData.id = "couldntrestore";
      errorData.msg = responseData.errorMessage === null ? "" : responseData.errorMessage;
      return false;
    }

    if (responseData.longUrl === null) {
      errorData.id = "invalid";
      return false;
    }

    return true;
  }

};
